package ru.dbotthepony.mc.otm.block.entity.worker;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.DoubleTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.core.Fraction;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public record MachineJob(ItemStack stack, double ticks, Fraction power, CompoundTag data) {
	public MachineJob(ItemStack stack, double ticks, Fraction power) {
		this(stack, ticks, power, new CompoundTag());
	}

	public MachineJob(ItemStack stack, double ticks) {
		this(stack, ticks, Fraction.ZERO);
	}

	public CompoundTag serializeNBT() {
		CompoundTag store_job = new CompoundTag();

		store_job.put("data", data);
		store_job.put("stack", stack.serializeNBT());
		store_job.putDouble("ticks", ticks);
		store_job.put("power", power.serializeNBT());

		return store_job;
	}

	@Nullable
	public static MachineJob deserializeNBT(@Nullable Tag nbt) {
		if (nbt == null)
			return null;

		if (nbt instanceof CompoundTag tag) {
			if (tag.get("stack") instanceof CompoundTag stack_tag && tag.get("ticks") instanceof DoubleTag ticks_processing_time && tag.contains("power")) {
				ItemStack stack = ItemStack.of(stack_tag);

				if (!stack.isEmpty()) {
					return new MachineJob(stack, ticks_processing_time.getAsDouble(), Fraction.deserializeNBT(tag.get("power")), tag.getCompound("data"));
				}
			}
		}

		return null;
	}
}
